// C Header File
// Created 8/24/2002; 12:45:42 PM
void UpdatePlayerOnMinimap(FATCONFIG *fc);
void DrawMinimap(FATCONFIG *fc);

#define ACCURACY_FOR_TURNING 12

//=============================================================================
// returns 0 if move would bring the character too near to a wall
//=============================================================================
short ValidMove(FATCONFIG* fc, short x, short y) {
    #define DIST_FROM_WALL (32 - ACCURACY_FOR_TURNING)

    // if we would come too near to a solid square...
    if ((*(fc->map_data+((y+DIST_FROM_WALL)>>6)*fc->map_width+((x+DIST_FROM_WALL)>>6))==1) ||
        (*(fc->map_data+((y-DIST_FROM_WALL)>>6)*fc->map_width+((x-DIST_FROM_WALL)>>6))==1) ||
        (*(fc->map_data+((y+DIST_FROM_WALL)>>6)*fc->map_width+((x-DIST_FROM_WALL)>>6))==1) ||
        (*(fc->map_data+((y-DIST_FROM_WALL)>>6)*fc->map_width+((x+DIST_FROM_WALL)>>6))==1))
    {
       return 0;        // false
    }
    return 1;           // true
}


//=============================================================================
// move player forward or backward depending on player's angle ...
//
// special player move routine (takes doors into account) AND don't allows
// the user to come too near to a wall, but try to slide automatically.
//=============================================================================
void PlayerMove(FATCONFIG* fc,int forward) {
    short stepx   = FAT_Cos16384(fc->cam_orientation) >> 11;
    short stepy   = FAT_Sin16384(fc->cam_orientation) >> 11;
    short newxpos = fc->cam_xpos + ((forward)? stepx: -stepx);
    short newypos = fc->cam_ypos + ((forward)? stepy: -stepy);
    short newxmod = newxpos >> 6;
    short newymod = newypos >> 6;

    //-------------------------------------------------------------------------
    // check if we are still within the map
    //-------------------------------------------------------------------------
    if (newxmod > fc->map_width-1  ||
        newymod > fc->map_height-1 ||
        newxmod < 0 || newymod < 0)
    {
       return;
    }

    if (!ValidMove(fc, newxpos, newypos)) {  // if new position is invalid ...
        // ...but old position is OK, we're against a wall to east or west
        if (ValidMove(fc, fc->cam_xpos, newypos)) {
            // don't change angle if movement is perpendicular to the wall
            if (stepy != 0) {
                // if sign(stepx)==sign(stepy), rotate clockwise = turn right
                if ((stepx>0)==(stepy>0)) PlayerTurn(fc,1,ANGLE_015/2);
                else                      PlayerTurn(fc,0,ANGLE_015/2);
                // still allow movement along y = parallel to wall
                fc->cam_ypos = newypos;
            }
        }
        // ...but (new x, old y) is OK, we're against a wall to north or south
        if (ValidMove(fc, newxpos, fc->cam_ypos)) {
            // don't change angle if movement is perpendicular to the wall
            if (stepx != 0) {
               // if sign(stepx)==sign(stepy), rotate counterclockwise = turn left
               if ((stepx>0)==(stepy>0)) PlayerTurn(fc,0,ANGLE_015/2);
               else                      PlayerTurn(fc,1,ANGLE_015/2);
               // still allow movement along x = parallel to wall
               fc->cam_xpos = newxpos;
            }
        }
        // ...otherwise we're in a corner, so do nothing
        return;
    }

    //-------------------------------------------------------------------------
    // everything okay -> let's set the new position
    //-------------------------------------------------------------------------
    fc->cam_xpos = newxpos;
    fc->cam_ypos = newypos;
}
//GameMode:
//0-Traditional
//1-Interactive
short HandlePlayerMovement(FATCONFIG *fc)
{
PADSTATES key = 0;
short moving_speed;
short turning_speed;
switch(gamedata.cfg.gamemode)
{
case 0:    



          for(moving_speed=0;moving_speed<MOVE_SPEED;moving_speed++)
           {
						if(!gamedata.turn_delay)
           	PlayerMove(fc,PADSTATE_UP);
           }
           if(gamedata.turn_delay)
           {
           	for(turning_speed=0;turning_speed<TURN_SPEED;turning_speed++)
           	PlayerTurn(fc,gamedata.turn_direction,ANGLE_015/2);
           	gamedata.turn_delay--;
           	if(gamedata.turn_flag) gamedata.turn_flag = 0;
           }	
if(gamedata.xcamcur!=fc->cam_xpos) // Player is moving east or west
{
	fc->cam_ypos = (fc->cam_ypos & ~63)+32;
}
else if(gamedata.ycamcur!=fc->cam_ypos) // Player is moving north or south
{
	fc->cam_xpos = (fc->cam_xpos & ~63)+32;
}



#if ACCURACY_FOR_TURNING <= MOVE_SPEED
#error ACCURACY_FOR_TURNING (HandlePlayerMovement(), player.c) must be >= MOVE_SPEED.
#endif

if(gamedata.turn_flag)
{
if(PACABS(((fc->cam_xpos & ~63)+32 - fc->cam_xpos))<ACCURACY_FOR_TURNING
 && PACABS(((fc->cam_ypos & ~63)+32 - fc->cam_ypos))<ACCURACY_FOR_TURNING)
{
      			if(!gamedata.turn_delay)
						{
						gamedata.turn_delay = (TURN_STEP)/TURN_SPEED;
						}    		
}	
}

        if ((key = FAT_GetPadState())) {
           if (key & PADSTATE_ESC)       return 0;
           if(key & PADSTATE_DOWN)
           {
						if(!gamedata.turn_delay)
						{
						gamedata.turn_delay = (TURN_STEP*2)/TURN_SPEED;// *2 because it's 180d
           gamedata.turn_direction = PADSTATE_RIGHT;	
						}
           }
           //if(key & PADSTATE_UP) gamedata.up_down_movement = 1;

           if (key & PADSTATE_LEFTRIGHT)
           {
						gamedata.turn_flag = 1;
						gamedata.turn_direction = key & PADSTATE_RIGHT;

            }
        }
break;
case 1:
       if ((key = FAT_GetPadState())) {
           if (key & PADSTATE_ESC)       return 0;
           if (key & PADSTATE_UPDOWN)
           {
           for(moving_speed=0;moving_speed<MOVE_SPEED;moving_speed++)
               PlayerMove(fc,key & PADSTATE_UP);
           }
           if (key & PADSTATE_LEFTRIGHT)
           {
           for(turning_speed=0;turning_speed<TURN_SPEED;turning_speed++)
            PlayerTurn(fc,key & PADSTATE_RIGHT,ANGLE_015/2);
            }
        }
break;
}       
return 1; //Esc wasn't pressed
}


short InitGhosts(FATCONFIG *fc);
void InitGUI(void);

void KillPlayer(FATCONFIG *fc)
{
	short a;
	unsigned long c;
	for(a=0;a<7;a++)
	{
	c = int5_counter;
		OSContrastUp();
		while(c == int5_counter);
		c = int5_counter;
		while(c == int5_counter);		
		c = int5_counter;
		while(c == int5_counter);			
	}
	
	fc->cam_xpos = 96;
	fc->cam_ypos = 96;
	fc->cam_orientation = 0;
	--gamedata.lives;
	count5_start = int5_counter;
	if(gamedata.lives<0) gamedata.quitting = -1;
	
	InitGhosts(fc);
	InitGUI();
	DrawMinimap(fc);
	
	
	for(a=0;a<7;a++)
	{
	c = int5_counter;
		OSContrastDn();
		while(c == int5_counter);
		c = int5_counter;
		while(c == int5_counter);		
		c = int5_counter;
		while(c == int5_counter);			
	}	
}


void PlayerGoToNextLevel(FATCONFIG *fc)
{
	short a;
	unsigned long c;
	for(a=0;a<7;a++)
	{
	c = int5_counter;
		OSContrastDn();
		while(c == int5_counter);
		c = int5_counter;
		while(c == int5_counter);		
		c = int5_counter;
		while(c == int5_counter);			
	}
	if(gamedata.level<(ext_level_data==NULL?LEVELCOUNT:1))
	{
	++gamedata.lives;
	++gamedata.level;
	count5_start = int5_counter;
	
	fc->cam_xpos = 96;
	fc->cam_ypos = 96;
	fc->cam_orientation = 0;
	fc->map_data = map[gamedata.level-1];
	SetGameVariables(fc);
	InitGUI();
	DrawMinimap(fc);
			
	}
else gamedata.quitting=WON_QUIT_LEVEL;
	for(a=0;a<7;a++)
	{
	c = int5_counter;
		OSContrastUp();
		while(c == int5_counter);
		c = int5_counter;
		while(c == int5_counter);		
		c = int5_counter;
		while(c == int5_counter);			
	}	
}


void CheckPlayerMinimapUpdate(FATCONFIG *fc)
{
	short x = fc->cam_xpos;
	short y = fc->cam_ypos;
	
	if((gamedata.xcamcur>>6!=x>>6)||(gamedata.ycamcur>>6!=y>>6))
	UpdatePlayerOnMinimap(fc);
	
	gamedata.xcamcur = x;
	gamedata.ycamcur = y;


}
